from dataclasses import dataclass, field, InitVar
from streng.tools.rccola import rcmain, rcinput
from streng.ppp.sections.concrete.reinforcement.combos import LongReinforcementLayer as LongLayer
from streng.ppp.sections.concrete.reinforcement.areas import As
from streng.ppp.sections.concrete.reinforcement.ratios import ρw_vol
import numpy as np
import matplotlib.pyplot as plt
import datetime

PI = np.pi


@dataclass
class ColumnRectangular:
    b: float
    h: float
    cover: float
    length: float
    # corner bars
    bars_corn_n: int  # = field(default=4)
    bars_corn_dia: float
    # εσωτερικές ράβδοι παρειών
    long_layer_main_internal: LongLayer  # εσωτερικές ράβδοι (κάθε μίας) κύριας παρειάς
    long_layer_sec_internal: LongLayer  # εσωτερικές ράβδοι (κάθε μίας) δευτερεύουσας παρειάς
    # εγκάρσιος οπλισμός
    trans_pattern: int
    trans_dia: float
    trans_s: float

    rc_in: rcinput.RccolaInput

    conc_fc: float = 24.
    steel_fy: float = 440.
    steel_Es: float = 200000.
    steel_εsh: float = 0.01
    steel_Esh: float = 0.
    steel_εsu: float = 0.10
    steel_fu: float = 550.
    steel_fy_trans: float = 440.
    steel_εsu_trans: float = 0.10

    check_side: str = field(default='long')  # long or short

    rcc: rcmain.Rccola = field(init=False)

    def __post_init__(self):
        # Διαστάσεις του πυρήνα
        self.hc = self.h - 2 * self.cover - self.trans_dia
        self.bc = self.b - 2 * self.cover - self.trans_dia

        # Υπολογίζω ότι ο οπλισμός της κύριας παρειάς έχει τα γωνικά σίδερα (2) και επιπλέον τα ενδιάμεσα σίδερα
        _ns, _dias = self.long_layer_main_internal.ns.copy(
        ), self.long_layer_main_internal.dias.copy()
        _ns.append(self.bars_corn_n)
        _dias.append(self.bars_corn_dia)
        self.long_layer_main_full = LongLayer(ns=_ns, dias=_dias)

        # Βρίσκω τις θέσεις όλων των διαμήκων ράβδων. tuples(x, y, dia)
        self.all_bars = []
        self.__get_bars_locations()

        # Υπολογίζω το ογκομετρικό ποσοστό των συνδετήρων
        self.ρw_vol = self.__get_volumetric_ratio_trans()

    
    def __get_volumetric_ratio_trans(self):
        TC = self.hc
        BC = self.bc

        if self.trans_pattern == 1:
            hoops_length = 2 * TC + 2 * BC
        elif self.trans_pattern == 2:
            hoops_length = 2 * TC + 3 * BC
        elif self.trans_pattern == 3:
            hoops_length = 2 * TC + 4 * BC + 2 * (TC / 3)
        elif self.trans_pattern == 4:
            hoops_length = 2 * TC + 5 * BC + 2 * (TC / 2)
        elif self.trans_pattern == 5:
            hoops_length = 3 * TC + 2 * BC
        elif self.trans_pattern == 6:
            hoops_length = 3 * TC + 3 * BC
        elif self.trans_pattern == 7:
            hoops_length = 3 * TC + 4 * BC + 2 * (TC / 3)
        elif self.trans_pattern == 8:
            hoops_length = 3 * TC + 5 * BC + 2 * (TC / 2)

        return ρw_vol(hoops_length, self.trans_dia, self.bc, self.hc, self.trans_s)


    def __get_bars_locations(self):
        cnom = self.cover
        dia_main = self.long_layer_main_full.dia_equiv
        n_main = self.long_layer_main_full.ntot
        dia_sec = self.long_layer_sec_internal.dia_equiv
        n_sec = self.long_layer_sec_internal.ntot + 2  # προσθέτω τις γωνιακές
        spacing_main = (self.b - 2 * (cnom + self.trans_dia) -
                        dia_main) / (n_main - 1)
        spacing_sec = (self.h - 2 * (cnom + self.trans_dia) -
                       dia_main) / (n_sec - 1)

        first_bar_xy = cnom + self.trans_dia + 0.5 * dia_main
        for i in range(n_main):
            self.all_bars.append(
                (first_bar_xy + i*spacing_main, first_bar_xy, dia_main))
            self.all_bars.append(
                (first_bar_xy + i*spacing_main, self.h - first_bar_xy, dia_main))
        for i in range(n_sec-2):
            self.all_bars.append(
                (first_bar_xy, first_bar_xy + spacing_sec + i*spacing_sec, dia_sec))
            self.all_bars.append(
                (self.b - first_bar_xy, first_bar_xy + spacing_sec + i*spacing_sec, dia_sec))

    def plot_section(self):
        ax = plt.gca()

        for bar in self.all_bars:
            circle = plt.Circle(
                (bar[0], bar[1]), radius=bar[2], fc='r', linewidth=1)
            plt.gca().add_patch(circle)

        section_rectangle = plt.Rectangle(
            (0, 0), self.b, self.h, 0, fc='grey', linewidth=2, edgecolor='black', fill=True, alpha=0.5)
        plt.gca().add_patch(section_rectangle)

        plt.axis('scaled')

        return ax

    def prepare_AnySection(self, filename):
        textlines = []
        textlines.append('area')
        textlines.append('    label conc')
        textlines.append(
            f'    material concrete_confinedKappos {self.conc_fc} 0.002 {self.ρw_vol} {self.bc} {self.trans_s} {self.steel_fy_trans} 1')
        textlines.append(f'    section rectangular {self.b:.3f} {self.h:.3f}')
        textlines.append('end area')
        textlines.append('')
        textlines.append('fibergroup')
        textlines.append('    label reinf')
        textlines.append(
            f'    material steel_ParkSampson {self.steel_Es} {self.steel_fy} {self.steel_fu} {self.steel_εsh} {self.steel_εsu}')
        for bar in self.all_bars:
            textlines.append(
                f'    bar {bar[0]:.3f} {bar[1]:.3f} d{1000*bar[2]:.2f}')
        textlines.append('end fibergroup')

        with open(filename, 'w') as f:
            f.write('\n'.join(textlines))

    def run_rccola(self):
        T = self.h
        B = self.b
        TC = self.hc
        BC = self.bc
        GT = TC - self.trans_dia - self.bars_corn_dia
        NB1 = 2 * self.long_layer_main_full.ntot
        AS1 = 10000. * self.long_layer_main_full.As_equiv
        NB2 = 2 * self.long_layer_sec_internal.ntot
        AS2 = 10000. * self.long_layer_sec_internal.As_equiv

        if self.trans_pattern == 1:
            NBAR = 0
            XSTRP = 2
            PPP = (2 * TC + 2 * BC) * As(self.trans_dia) / \
                (TC * BC * self.trans_s)
        elif self.trans_pattern == 2:
            NBAR = 0
            XSTRP = 2
            PPP = (2 * TC + 3 * BC) * As(self.trans_dia) / \
                (TC * BC * self.trans_s)
        elif self.trans_pattern == 3:
            NBAR = 1
            XSTRP = 2
            PPP = (2 * TC + 4 * BC + 2 * (TC / 3)) * \
                As(self.trans_dia) / (TC * BC * self.trans_s)
        elif self.trans_pattern == 4:
            NBAR = 1
            XSTRP = 2
            PPP = (2 * TC + 5 * BC + 2 * (TC / 2)) * \
                As(self.trans_dia) / (TC * BC * self.trans_s)
        elif self.trans_pattern == 5:
            NBAR = 0
            XSTRP = 3
            PPP = (3 * TC + 2 * BC) * As(self.trans_dia) / \
                (TC * BC * self.trans_s)
        elif self.trans_pattern == 6:
            NBAR = 1
            XSTRP = 3
            PPP = (3 * TC + 3 * BC) * As(self.trans_dia) / \
                (TC * BC * self.trans_s)
        elif self.trans_pattern == 7:
            NBAR = 1
            XSTRP = 3
            PPP = (3 * TC + 4 * BC + 2 * (TC / 3)) * \
                As(self.trans_dia) / (TC * BC * self.trans_s)
        elif self.trans_pattern == 8:
            NBAR = 1
            XSTRP = 3
            PPP = (3 * TC + 5 * BC + 2 * (TC / 2)) * \
                As(self.trans_dia) / (TC * BC * self.trans_s)

        print('PPP = ', PPP)
        print('PPP2 = ', self.ρw_vol)

        col_input = rcinput.RccolaInput

        col_input.first_row = rcinput.FirstRow(TYPEC=1,
                                               TYPES=1,
                                               ISPALL=0,
                                               ICSTR=8,
                                               ICSTRS=0,
                                               NPTS=0,
                                               NPTSS=0,
                                               ITCONF=1,
                                               ISSTL=0,
                                               IECU=1,
                                               ISSTR=0,
                                               IGRAPH=3)

        col_input.second_row = rcinput.SecondRow(NLOAD=0,
                                                 NLAYER=0,
                                                 NDATA1=0,
                                                 NDATA2=0,
                                                 NSTRNI=24,
                                                 NSTRNC=24,
                                                 NITT=30)

        col_input.conc_sect_info = rcinput.ConcSectInfo(T, B, TC, BC)

        col_input.reinf_place_info_TYPES12 = rcinput.ReinfPlaceInfoTYPES12(GT=GT,
                                                                           NB1=NB1,
                                                                           AS1=AS1,
                                                                           NB2=NB2,
                                                                           AS2=AS2,
                                                                           NBAR=NBAR,
                                                                           AF=None)

        col_input.long_reinf_props = rcinput.LongReinfProps(FYS=self.steel_fy,
                                                            YMS=self.steel_Es,
                                                            ESH=self.steel_εsh,
                                                            YSH=self.steel_Esh,
                                                            ESU=self.steel_εsu,
                                                            FSU=self.steel_fu,
                                                            ESDYN=None)

        col_input.trans_reinf_info = rcinput.TransReinfInfo(FYSTRP=self.steel_fy_trans,
                                                            ASSTRP=XSTRP * 10000. * As(self.trans_dia),
                                                            S=self.trans_s,
                                                            XL=self.length,
                                                            XSTRP=2.000,
                                                            ESUSTRP=self.steel_εsu_trans)

        col_input.conc_props_ICSTR2348 = rcinput.ConcPropsICSTR2348(self.conc_fc,
                                                                    S=self.trans_s,
                                                                    PPP=PPP)

        # col_input.strains = np.array(
        #     [0.0, 0.001, 0.002, 0.003, 0.0035, 0.004, 0.005, 0.006, 0.007, 0.008, 0.009, 0.01, 0.011, 0.012,
        #      0.013, 0.014, 0.015, 0.016, 0.018, 0.02, 0.022, 0.024, 0.026, 0.028, 0.03])

        col_input.strains = np.array(
            [0.0, 0.0005, 0.0015, 0.002, 0.0025, 0.003, 0.0035, 0.004, 0.0045, 0.005, 0.006, 0.007, 0.008, 0.009, 0.01, 0.015, 0.02, 0.025, 0.03, 0.035, 0.04, 0.045, 0.05, 0.06, 0.07])



            #     ActiveStrains = New Single() {0, 0.0005, 0.0015, 0.002, 0.0025, 0.003, 0.0035, _
            #                             0.004, 0.0045, 0.005, 0.006, 0.007, 0.008, 0.009, _
            #                             0.01, 0.015, 0.02, 0.025, 0.03, 0.035, 0.04, 0.045, 0.05}
            #     ActiveStrNforXxpmax = 5
            # Case Is = 2
            #     ActiveStrains = New Single() {0, 0.00025, 0.0005, 0.00075, 0.001, 0.00125, 0.0015, 0.00175, 0.002, 0.00225, 0.0025, 0.00275, 0.003, 0.00325, 0.0035, 0.00375, _
            #                                      0.004, 0.00425, 0.0045, 0.00475, 0.005, 0.00525, 0.0055, 0.00575, 0.006, 0.00625, 0.0065, 0.00675, 0.007, 0.00725, 0.0075, 0.00775, _
            #                                      0.008, 0.00825, 0.0085, 0.00875, 0.009, 0.00925, 0.0095, 0.00975, 0.01, 0.01025, _
            #                                      0.0105, 0.01075, 0.011, 0.01125, 0.0115, 0.01175, 0.012, 0.01225, 0.0125, 0.01275, 0.013, 0.01325, 0.0135, 0.01375, 0.014, 0.01425, 0.0145, 0.01475, 0.015, _
            #                                      0.016, 0.017, 0.018, 0.019, 0.02, 0.025, 0.03, 0.035, 0.04, 0.045, 0.05}
            #     ActiveStrNforXxpmax = 12
            # Case Else
        col_input.axials = None

        self.rcc = rcmain.Rccola(input=col_input,
                            outfilename='pyrcout_square.txt')

        self.rcc.calc()
        self.rcc.write_out()
        # print(rcc.outtxt)



        self.rc_in = col_input

        # self.rc_in = rcinput.RccolaInput(conc_sect_info = conc_sect_info)




# plt.axes()

# circle = plt.Circle((0, 0), radius=0.75, fc='y')
# plt.gca().add_patch(circle)

# plt.axis('scaled')
# plt.show()

# def multiple_custom_plots(x, y, ax=None, plt_kwargs={}, sct_kwargs={}):
#     if ax is None:
#         ax = plt.gca()
#     ax.plot(x, y, **plt_kwargs)
#     ax.scatter(x, y, **sct_kwargs)
#     return(ax)

# plot_params = {'linewidth': 2, 'c': 'g', 'linestyle':'--'}
# scatter_params = {'c':'red', 'marker':'+', 's':100}
# xdata = [1, 2]
# ydata = [10, 20]

# # plt.figure(figsize=(10, 5))
# axxx = multiple_custom_plots(xdata, ydata, plt_kwargs=plot_params, sct_kwargs=scatter_params)
# plt.savefig('plot')
# # plt.show()



'''
examples from the docs page
http://docs.xlwings.org/en/stable/udfs.html#array-formulas-with-numpy-and-pandas
'''

import xlwings as xw
import numpy as np
import pandas as pd

@xw.func
def double_sum(x, y):
    """Returns twice the sum of the two arguments"""
    return 2 * (x + y)


@xw.func
def add_one(data):
    return [[cell + 1 for cell in row] for row in data]



# To force Excel to always give you a two-dimensional array, no matter whether the argument is a single cell,
# a column/row or a two-dimensional Range, you can extend the above formula like this:
@xw.func
@xw.arg('data', ndim=2)
def add_one(data):
    return [[cell + 1 for cell in row] for row in data]



@xw.func
@xw.arg('x', np.array, ndim=2)
@xw.arg('y', np.array, ndim=2)
def matrix_mult(x, y):
    return x @ y



@xw.func
@xw.arg('x', pd.DataFrame, index=False, header=False)
@xw.ret(index=False, header=False)
def CORREL2(x):
    """Like CORREL, but as array formula for more than 2 data sets"""
    return x.corr()


@xw.func
@xw.arg('x', pd.DataFrame)
@xw.ret(index=False)
def myfunction(x):
   # x is a DataFrame, do something with it
   return x


import numpy as np

@xw.func
@xw.ret(expand='table')
def dynamic_array(r, c):
    return np.random.randn(int(r), int(c))





"""
EC8 spectra functions

    .. uml::

        class spectra <<(M,#FF7700)>> {
        .. functions ..
        + S(ground_type, spectrum_type)
        + SDe(T, Se)
        + Sd(T, αg, S, TB, TC, TD, q, β=0.2)
        + Se(T, αg, S, TB, TC, TD, η=1.0)
        + TB(ground_type, spectrum_type)
        + TC(ground_type, spectrum_type)
        + TD(ground_type, spectrum_type, national_annex='default')
        + dg(αg, S, TC, TD)
        + αg(αgR, γI)
        + η(ξ)
        }
"""

import numpy as np
import math


def αg(αgR, γI):
    """

    Args:
        αgR (float): reference peak ground acceleration on type A ground
        γI: (float): importance factor

    Returns:
        float: design ground acceleration on type A ground

    """
    return αgR * γI


def S(ground_type, spectrum_type):
    """

    Args:
        ground_type (str): Ground type (A, B, C, D or E)
        spectrum_type (int): Spectrum type 1 or 2

    Returns:
        float: soil factor

    """
    data = {}
    if spectrum_type == 1:
        data = {'A': 1.0, 'B': 1.2, 'C': 1.15, 'D': 1.35, 'E': 1.4}
    elif spectrum_type == 2:
        data = {'A': 1.0, 'B': 1.35, 'C': 1.5, 'D': 1.8, 'E': 1.6}
    return data[ground_type]


def TB(ground_type, spectrum_type):
    """

    Args:
        ground_type (str): Ground type (A, B, C, D or E)
        spectrum_type (int): Spectrum type 1 or 2

    Returns:
        float: The lower limit of the period of the constant spectral acceleration branch

    """
    data = {}
    if spectrum_type == 1:
        data = {'A': 0.15, 'B': 0.15, 'C': 0.20, 'D': 0.20, 'E': 0.15}
    elif spectrum_type == 2:
        data = {'A': 0.05, 'B': 0.05, 'C': 0.10, 'D': 0.10, 'E': 0.05}
    return data[ground_type]


def TC(ground_type, spectrum_type):
    """

    Args:
        ground_type (str): Ground type (A, B, C, D or E)
        spectrum_type (int): Spectrum type 1 or 2

    Returns:
        float: The upper limit of the period of the constant spectral acceleration branch

    """
    data = {}
    if spectrum_type == 1:
        data = {'A': 0.40, 'B': 0.50, 'C': 0.60, 'D': 0.80, 'E': 0.5}
    elif spectrum_type == 2:
        data = {'A': 0.25, 'B': 0.25, 'C': 0.25, 'D': 0.30, 'E': 0.25}
    return data[ground_type]


def TD(ground_type, spectrum_type, national_annex='default'):
    """

    Args:
        ground_type (str): Ground type (A, B, C, D or E)
        spectrum_type (int): Spectrum type 1 or 2
        national_annex (str): Country national annex. Options are: 'default', 'greek'

    Returns:
        float: The value defining the beginning of the constant displacement response range of the spectrum

    """
    data = {}

    if national_annex == 'greek':
        if spectrum_type == 1:
            data = {'A': 2.50, 'B': 2.50, 'C': 2.50, 'D': 2.50, 'E': 2.50}
    else:
        if spectrum_type == 1:
            data = {'A': 2.00, 'B': 2.00, 'C': 2.00, 'D': 2.00, 'E': 2.00}

    if spectrum_type == 2:
        data = {'A': 1.20, 'B': 1.20, 'C': 1.20, 'D': 1.20, 'E': 1.20}
    return data[ground_type]


def Se(T, αg, S, TB, TC, TD, η=1.0):
    """

    Args:
        T (float): The vibration period of a linear single-degree-of-freedom system
        αg (float): The design ground acceleration on type A ground (ag = γI*agR)
        S (float): The soil factor
        TB (float): The lower limit of the period of the constant spectral acceleration branch
        TC (float): The upper limit of the period of the constant spectral acceleration branch
        TD (float): The value defining the beginning of the constant displacement response range of the spectrum
        η (float): The damping correction factor with a reference value of η = 1 for 5% viscous damping

    Returns:
        float: The elastic acceleration response spectrum. Given using the expressions:

        .. math::
            :nowrap:

            \\begin{eqnarray}
                0 \le T \le T_B \\rightarrow & S_e(T) & = α_g\cdot S \cdot (1+\dfrac{T}{T_B}\cdot(η\cdot 2.5 -1)) \\\\
                T_B \le T \le T_C \\rightarrow & S_e(T) & = α_g\cdot S \cdot η\cdot 2.5 \\\\
                T_C \le T \le T_D \\rightarrow & S_e(T) & = α_g\cdot S \cdot η\cdot 2.5\cdot \dfrac{T_C}{T} \\\\
                T_D \le T \le 4s \\rightarrow & S_e(T) & = α_g\cdot S \cdot η\cdot 2.5\cdot \dfrac{T_C\cdot T_D}{T^2}
            \\end{eqnarray}

    """

    condlist = [T <= TB,
                T <= TC,
                T <= TD,
                T <= 4]
    choicelist = [αg * S * (1.0 + (T / TB) * (η * 2.5 - 1)),
                  αg * S * η * 2.5,
                  αg * S * η * 2.5 * (TC / T),
                  αg * S * η * 2.5 * (TC * TD / T ** 2)]
    return np.select(condlist, choicelist)

    # if T <= TB:
    #     _Sd = αg * S * (1.0 + (T / TB) * (η * 2.5 - 1))
    # elif T <= TC:
    #     _Sd = αg * S * η * 2.5
    # elif T <= TD:
    #     _Sd = αg * S * η * 2.5 * (TC / T)
    # elif T <= 4:
    #     _Sd = αg * S * η * 2.5 * (TC * TD / T ** 2)
    # else:
    #     _Sd = 0
    #
    # return _Sd


def SDe(T, Se):
    """

    Args:
        T (float): The vibration period of a linear single-degree-of-freedom system
        Se (float):The elastic acceleration response spectrum

    Returns:
        float: The elastic displacement response spectrum. Given using the expression:

        .. math::
            S_{De}=S_e(T)\cdot(\dfrac{T}{2π})^2

    """
    return Se * (T / (2 * math.pi)) ** 2


def dg(αg, S, TC, TD):
    """

    Args:
        αg (float): The design ground acceleration on type A ground (ag = γI*agR)
        S (float): The soil factor
        TC (float): The upper limit of the period of the constant spectral acceleration branch
        TD (float): The value defining the beginning of the constant displacement response range of the spectrum

    Returns:
        float: Design ground displacement. Given using the expression:

         .. math::
            d_{g}=0.025\cdot α_g \cdot S \cdot T_C  \cdot T_D

    """
    return 0.025 * αg * S * TC * TD


def Sd(T, αg, S, TB, TC, TD, q, β=0.2):
    """

    Args:
        T (float): The vibration period of a linear single-degree-of-freedom system
        αg (float): The design ground acceleration on type A ground (ag = γI*agR)
        S (float): The soil factor
        TB (float): The lower limit of the period of the constant spectral acceleration branch
        TC (float): The upper limit of the period of the constant spectral acceleration branch
        TD (float): The value defining the beginning of the constant displacement response range of the spectrum
        q (float): The behaviour factor
        β (float): The lower bound factor for the horizontal design spectrum. Recommended value for β is 0.2

    Returns:
        float: Design spectrum for elastic analyses. Given using the expressions:

        .. math::
            :nowrap:

            \\begin{eqnarray}
                0 \le T \le T_B \\rightarrow & S_d(T) & = α_g\cdot S \cdot (\dfrac{2}{3}+\dfrac{T}{T_B}\cdot(\dfrac{2.5}{q} - \dfrac{2}{3})) \\\\
                T_B \le T \le T_C \\rightarrow & S_d(T) & = α_g\cdot S \cdot \dfrac{2.5}{q} \\\\
                T_C \le T \le T_D \\rightarrow & S_d(T) & = α_g\cdot S \cdot \dfrac{2.5}{q} \cdot \dfrac{T_C}{T} \ge β \cdot α_g \\\\
                T_D \le T \le 4s \\rightarrow & S_d(T) & = α_g\cdot S \cdot \dfrac{2.5}{q} \cdot \dfrac{T_C\cdot T_D}{T^2} \ge β \cdot α_g
            \\end{eqnarray}

    """

    condlist = [T <= TB,
                T <= TC,
                T <= TD,
                T <= 4]
    choicelist = [αg * S * (2.0 / 3.0 + (T / TB) * (2.5 / q - 2.0 / 3.0)),
                  αg * S * 2.5 / q,
                  np.maximum(αg * S * (2.5 / q) * (TC / T), β * αg),
                  np.maximum(αg * S * (2.5 / q) * (TC * TD / T ** 2), β * αg)]
    return np.select(condlist, choicelist)

    # if T <= TB:
    #     _Sd = αg * S * (2.0 / 3.0 + (T / TB) * (2.5 / q - 2.0 / 3.0))
    # elif T <= TC:
    #     _Sd = αg * S * 2.5 / q
    # elif T <= TD:
    #     _Sd = max(αg * S * (2.5 / q) * (TC / T), β * αg)
    # elif T <= 4:
    #     _Sd = max(αg * S * (2.5 / q) * (TC * TD / T ** 2), β * αg)
    # else:
    #     _Sd = 0
    #
    # return _Sd


def η(ξ):
    """

    Args:
        ξ (float): the viscous damping ratio of the structure[%]

    Returns:
        float: The value of the damping correction factor. Given using the expressions:

        .. math::
            η = \sqrt{\dfrac{10}{5+ξ}} \ge 0.55
    """

    return max(0.55, (10. / (5. + ξ)) ** 0.5)



import xlwings as xw
import numpy as np
import pandas as pd

from ...ppp.loads.slabs_to_beams import SlabLoadsToBeams


@xw.func
def xwf_ppp_loads_slabs_to_beams(lmax, lmin, slabtype, slabload, beam):
    sl = SlabLoadsToBeams(l_max = lmax,
                          l_min = lmin,
                          slab_type = slabtype,
                          slab_load = slabload)
    return sl.beams_loads[beam]





"""
    .. uml::

        class lists_and_dicts_and_pandas_etc <<(M,#FF7700)>> {
        .. functions ..
        + area_under_curve(x, y, x_start, x_end)
        + xy_with_endpoints(x, y, x_start, x_end)
        + intersection(x1, y1, x2, y2)
        - _rect_inter_inner(x1, x2)
        - _rectangle_intersection_(x1,y1,x2,y2)
        }
"""

import numpy as np


def distribute_integers(oranges, plates):
    base, extra = divmod(oranges, plates)
    return [base + (i < extra) for i in range(plates)]


def area_under_curve(x, y, x_start, x_end):
    x_new, y_new = xy_with_endpoints(x, y, x_start, x_end)
    res = np.trapz(y=y_new, x=x_new)
    return res


def xy_with_endpoints(x, y, x_start, x_end):
    if x_start < x[0]:
        x_start = x[0]

    if x_end > x[-1]:
        x_end = x[-1]

    # Για την αρχή
    y_start = float(np.interp(x_start, x, y))
    # Κρατώ μόνο τις τιμές μετά από το x_start
    _x = x[(x > x_start)]
    _y = y[len(y) - len(_x):len(y)]
    # Προσθέτω τα x, y της παρεμβολής
    _x = np.append(x_start, _x)
    _y = np.append(y_start, _y)

    # Για το τέλος
    y_end = float(np.interp(x_end, x, y))
    # Κρατώ μόνο τις τιμές μέχρι μία πρίν από το x_end
    _x = _x[(_x < x_end)]
    _y = _y[0:len(_x)]
    # Προσθέτω τα x, y της παρεμβολής
    _x = np.append(_x, x_end)
    _y = np.append(_y, y_end)

    # Εναλλακτικά με παρεμβολή σε όλες τις τιμές των y...μάλλον όχι για καμπύλες με δοντάκια
    # ....μικρές διαφορές όμως
    # _x = x[(x >= x_start)]
    # print(_x)
    # print(y)
    # _x = _x[(_x <= x_end)]
    # _x = np.append(_x, x_end)
    # print(_x)
    # _y = np.interp(_x, x, y)

    return _x, _y


def _rect_inter_inner(x1, x2):
    n1 = x1.shape[0] - 1
    n2 = x2.shape[0] - 1
    X1 = np.c_[x1[:-1], x1[1:]]
    X2 = np.c_[x2[:-1], x2[1:]]
    S1 = np.tile(X1.min(axis=1), (n2, 1)).T
    S2 = np.tile(X2.max(axis=1), (n1, 1))
    S3 = np.tile(X1.max(axis=1), (n2, 1)).T
    S4 = np.tile(X2.min(axis=1), (n1, 1))
    return S1, S2, S3, S4


def _rectangle_intersection_(x1, y1, x2, y2):
    S1, S2, S3, S4 = _rect_inter_inner(x1, x2)
    S5, S6, S7, S8 = _rect_inter_inner(y1, y2)

    C1 = np.less_equal(S1, S2)
    C2 = np.greater_equal(S3, S4)
    C3 = np.less_equal(S5, S6)
    C4 = np.greater_equal(S7, S8)

    ii, jj = np.nonzero(C1 & C2 & C3 & C4)
    return ii, jj


def intersection(x1, y1, x2, y2):
    """Copied from https://github.com/sukhbinder/intersection

        INTERSECTIONS Intersections of curves.
        Computes the (x,y) locations where two curves intersect.  The curves
        can be broken with NaNs or have vertical segments.

        .. usage:
            x,y=intersection(x1,y1,x2,y2)
            Example:
                a, b = 1, 2
                phi = np.linspace(3, 10, 100)
                x1 = a*phi - b*np.sin(phi)
                y1 = a - b*np.cos(phi)
                x2=phi
                y2=np.sin(phi)+2
                x,y=intersection(x1,y1,x2,y2)
                plt.plot(x1,y1,c='r')
                plt.plot(x2,y2,c='g')
                plt.plot(x,y,'*k')
                plt.show()
    """
    ii, jj = _rectangle_intersection_(x1, y1, x2, y2)
    n = len(ii)

    dxy1 = np.diff(np.c_[x1, y1], axis=0)
    dxy2 = np.diff(np.c_[x2, y2], axis=0)

    T = np.zeros((4, n))
    AA = np.zeros((4, 4, n))
    AA[0:2, 2, :] = -1
    AA[2:4, 3, :] = -1
    AA[0::2, 0, :] = dxy1[ii, :].T
    AA[1::2, 1, :] = dxy2[jj, :].T

    BB = np.zeros((4, n))
    BB[0, :] = -x1[ii].ravel()
    BB[1, :] = -x2[jj].ravel()
    BB[2, :] = -y1[ii].ravel()
    BB[3, :] = -y2[jj].ravel()

    for i in range(n):
        try:
            T[:, i] = np.linalg.solve(AA[:, :, i], BB[:, i])
        except:
            T[:, i] = np.NaN

    in_range = (T[0, :] >= 0) & (T[1, :] >= 0) & (T[0, :] <= 1) & (T[1, :] <= 1)

    xy0 = T[2:, in_range]
    xy0 = xy0.T
    return xy0[:, 0], xy0[:, 1]



import xlwings as xw
from dataclasses import dataclass
from streng.xwflib.xwf_main import *
from streng.ppp.loads.slabs_to_beams import SlabLoadsToBeams


@dataclass
class slabs_phd:
    typos: str
    Lmax: float
    Lmin: float


# Πλαισιακοί φορείς
slabs_frames = {}
slabs_frames['Π1'] = slabs_phd(typos = '5a',
                              Lmax = 6.0, Lmin = 3.0)
slabs_frames['Π2'] = slabs_phd(typos = '6',
                              Lmax = 4.0, Lmin = 3.0)
slabs_frames['Π3'] = slabs_frames['Π1']                            


# Δίδυμοι φορείς 59 - 9ώροφα
slabs_duals9 = {}
slabs_duals9['Π1'] = slabs_phd(typos = '4',
                              Lmax = 6.0, Lmin = 3.0)
slabs_duals9['Π2'] = slabs_phd(typos = '5b',
                              Lmax = 6.0, Lmin = 3.0)
slabs_duals9['Π3'] = slabs_duals9['Π1']    

slabs_duals9['Π4'] = slabs_phd(typos = '5a',
                              Lmax = 6.0, Lmin = 3.0)
slabs_duals9['Π5'] = slabs_phd(typos = '6',
                              Lmax = 6.0, Lmin = 3.0)
slabs_duals9['Π6'] = slabs_duals9['Π4']   

slabs_duals9['Π7'] = slabs_duals9['Π4']
slabs_duals9['Π8'] = slabs_duals9['Π5']
slabs_duals9['Π9'] = slabs_duals9['Π7']


# Δίδυμοι φορείς 59 - 4ώροφα
slabs_duals4 = {}
slabs_duals4['Π1'] = slabs_phd(typos = '4',
                              Lmax = 4.5, Lmin = 4.0)
slabs_duals4['Π2'] = slabs_phd(typos = '5b',
                              Lmax = 4.5, Lmin = 4.0)
slabs_duals4['Π3'] = slabs_duals4['Π1']                                

slabs_duals4['Π4'] = slabs_phd(typos = '5a',
                              Lmax = 4.5, Lmin = 4.0)
slabs_duals4['Π5'] = slabs_phd(typos = '6',
                              Lmax = 4.5, Lmin = 4.0)
slabs_duals4['Π6'] = slabs_duals4['Π4'] 

slabs_duals4['Π7'] = slabs_duals4['Π4']
slabs_duals4['Π8'] = slabs_duals4['Π5']
slabs_duals4['Π9'] = slabs_duals4['Π7']


@xw.func
def phd_beam_loads_from_slabs(building_type, slab_beam_combo, slab_load):
    _beam_load = 0

    if slab_beam_combo != '':
        # print(slab_beam_combo + str(type(slab_beam_combo)))

        slab_beams = slab_beam_combo.split(',')

        for slab_beam in slab_beams:
            sl_b = slab_beam.split(':')
            sl_name, sl_beam_checked = sl_b[0], int(sl_b[1])
            if building_type == 'frames':
                sl = slabs_frames[sl_name]
            elif building_type == 'duals4':
                sl = slabs_duals4[sl_name]
            elif building_type == 'duals9':
                sl = slabs_duals9[sl_name]
            else:
                sl=None

            sltb = SlabLoadsToBeams(l_max = sl.Lmax,
                                    l_min = sl.Lmin,
                                    slab_type = sl.typos,
                                    slab_load = slab_load)
            _beam_load += sltb.beams_loads[sl_beam_checked]
    return _beam_load



   





"""
Beams effective width

    .. uml::

        class effective_width <<(M,#FF7700)>> {
        .. functions ..
        + beff(bw, beff1, beff2, b)
        + beffi(bi, l0)
        + l0(l1=0, l2=0, l3=0, zero_moments_case=0)
        }

"""


def beff(bw, beff1, beff2, b):
    """ Effective flange width

        .. image paths are relative to the shpinx source folder that calls this automodule/autoclass

        .. image:: ../../img/ec_beff.png


    Args:
        bw (float): Width of the web
        beff1 (float): Side 1 effective flange width
        beff2 (float): Side 2 effective flange width
        b (float): bw + b1 + b2

    Returns:
        float: Given using the expression:

        .. math::
            b_{eff}=\sum{b_{eff,i}} + b_w \le b

    """
    return min(bw + beff1 + beff2, b)


def beffi(bi, l0):
    """ Side i effective flange width

        .. image:: ../../img/ec_beff_l0.png

    Args:
        bi (float): half net length between adjacent beams
        l0 (float): Distance between points of zero moment

    Returns:
        float: Given using the expression:

        .. math::
            b_{eff,i}=0.2\cdot b_i +0.1\cdot l_0 \le 0.2\cdot l_0

    """
    return min(0.2 * bi + 0.1 * l0, 0.2 * l0, bi)


def l0(l1=0, l2=0, l3=0, zero_moments_case=0):
    """ Distance between points of zero moment

        .. image:: ../../img/ec_beff_l0.png

    Args:
        l1 (float): μήκος αμφιέρειστης ή ακραίου ανοίγματος
        l2 (float): μήκος μεσαίουν ανοίγματος
        l3 (float): μήκος προβόλου
        zero_moments_case (int): συνθήκες στήριξης. 0: αμφιέρειστη, 1: ακραίο άνοιγμα, 2: μεσαίο άνοιγμα, 3: μεσαία στήριξη, 4: στήριξη προβόλου

    Returns:
        float: Υποπεριπτώσεις σύμφωνα με το σχήμα


    """
    _l0 = 0
    if zero_moments_case == 0: _l0 = 1.00 * l1  # αμφιέρειστη
    if zero_moments_case == 1: _l0 = 0.85 * l1  # ακραίο άνοιγμα
    if zero_moments_case == 2: _l0 = 0.70 * l2  # μεσαίο άνοιγμα
    if zero_moments_case == 3: _l0 = 0.15 * (l1 + l2)  # μεσαία στήριξη
    if zero_moments_case == 4: _l0 = 0.15 * l2 + l3  # στήριξη προβόλου

    return _l0



from setuptools import setup, find_packages
from os import path
from io import open


# with open("README.rst", "r", encoding='utf-8') as fh:
#     long_description = fh.read()

with open('LICENSE', "r", encoding='utf-8') as f:
    license = f.read()

# LONG_DESCRIPTION = """\
# Seaborn is a library for making statistical graphics in Python. It is built on top of `matplotlib <https://matplotlib.org/>`_ and closely integrated with `pandas <https://pandas.pydata.org/>`_ data structures.
# Here is some of the functionality that seaborn offers:
# - A dataset-oriented API for examining relationships between multiple variables
# - Specialized support for using categorical variables to show observations or aggregate statistics
# - Options for visualizing univariate or bivariate distributions and for comparing them between subsets of data
# - Automatic estimation and plotting of linear regression models for different kinds dependent variables
# - Convenient views onto the overall structure of complex datasets
# - High-level abstractions for structuring multi-plot grids that let you easily build complex visualizations
# - Concise control over matplotlib figure styling with several built-in themes
# - Tools for choosing color palettes that faithfully reveal patterns in your data
# Seaborn aims to make visualization a central part of exploring and understanding data. Its dataset-oriented plotting functions operate on dataframes and arrays containing whole datasets and internally perform the necessary semantic mapping and statistical aggregation to produce informative plots.
# """

setup(
    version='0.0.3',
    name='streng',
    description='structural engineering tools with python',
    # long_description=LONG_DESCRIPTION,
    # long_description_content_type="text/markdown",
    author='panagop',
    author_email="giorgos@panagop.com",
    url=r'https://github.com/gpanagop/streng',
    license=license,
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
    packages=find_packages(exclude=('testing', 'docs')),
    install_requires=[
        'tabulate', 'numpy', 'matplotlib', 'pandas', 'sympy', 'scipy', 'sqlalchemy'],
)


"""
Shear functions

    .. uml::

        class shear <<(M,#FF7700)>> {
        .. functions ..
        + VRdc(CRdc, Asl, fck, σcp, bw, d, units='N-mm')
        + VRdmax(bw, d, fck, fyk, fywk, θ, αcw=1.0, γc=1.5, units='N-mm-rad')
        + VRds(nw, diaw, d, fyk, fywk, θ, s, γs=1.15, units='N-mm-rad')
        }
"""

import math


def VRdc(CRdc, Asl, fck, σcp, bw, d, units='N-mm'):
    """ The design value for the shear resistance :math:`V_{Rd,c}` [N]

    Units are 'N-mm', unless specified otherwise. Alternative options are: 'kN-m'

    .. highlight:: python
    .. code-block:: python
       :linenos:

        vrdc = shear.VRdc(CRdc=0.12,
                               Asl=308,
                               fck=20.,
                               σcp=0.66667,
                               bw=250.,
                               d=539.,
                               units='N-mm')

    Args:
        CRdc (float): 0.18/γc
        Asl (float): [mm2] is the area of the tensile reinforcement
        fck (float): [N/mm2]
        σcp (float):  :math:`σ_{cp}=N_{Ed}/A_c \lt 0.2f_{cd}` [N/mm2]
        bw (float): The smallest width of the cross-section in the tensile area. [mm]
        d (float): Effective depth of the cross-section. [mm]

    Returns:
        VRdc (dict):

        Result value in [N], unless specified otherwise. Intermediate results are always 'N-mm'

        Given using the expressions

        .. math::
            V_{Rd,c} = max \\left\{\\begin{matrix}
            [C_{Rd,c} \cdot k \cdot(100\cdot ρ_l\cdot f_{ck})^{1/3} + k_1 \cdot σ_{cp}] \cdot b_w \cdot d \\\\
            (v_{min} + k_1 \cdot σ_{cp}) \cdot b_w \cdot d
            \\end{matrix}\\right.

        where:

        - :math:`k=1 + \sqrt{\dfrac{200}{d}} <= 2.0`

        - :math:`ρ_l=\dfrac{A_{sl}}{b_w \cdot d}<=0.02`

        - :math:`k_1 = 0.15`

    """
    _VRdc = {}

    if units == 'N-mm':
        pass
    elif units == 'kN-m':
        Asl = Asl * 10 ** 6
        fck *= 0.001
        σcp *= 0.001
        bw *= 1000
        d *= 1000
    else:
        pass

    ρl = min(Asl / (bw * d), 0.02)
    k = min(1 + (200.0 / d) ** 0.5, 2.0)
    vmin = 0.035 * k ** 1.5 * fck ** 0.5
    k1 = 0.15

    VRdc1 = (CRdc * k * math.pow((100 * ρl * fck), (1 / 3)) + k1 * σcp) * bw * d
    VRdc2 = (vmin + k1 * σcp) * bw * d

    _VRdc['ρl'] = ρl
    _VRdc['k'] = k
    _VRdc['vmin'] = vmin
    _VRdc['k1'] = k1
    _VRdc['VRdc1'] = VRdc1
    _VRdc['VRdc2'] = VRdc2
    _VRdc['value'] = max(VRdc1, VRdc2)

    if units == 'N-mm':
        pass
    elif units == 'kN-m':
        _VRdc['value'] *= 0.001
    else:
        pass

    return _VRdc


def VRdmax(bw, d, fck, fyk, fywk, θ, αcw=1.0, γc=1.5, units='N-mm-rad'):
    """ Maximum value for the shear resistance :math:`V_{Rd,max}` [N]

    Units are 'N-mm', unless specified otherwise. Alternative options are: 'kN-m-rad'

    .. highlight:: python
    .. code-block:: python
       :linenos:

        vrdmax = shear.VRdmax(bw=250.,
                              d=539.,
                              fck=20.,
                              fyk=500.,
                              fywk=500.,
                              θ=np.pi/4,
                              αcw = 1.0,
                              γc = 1.5,
                              units='N-mm-rad')

    Args:
        bw (float): The smallest width of the cross-section in the tensile area. [mm]
        d (float): Effective depth of the cross-section. [mm]
        fck (float): [N/mm2]
        fyk (float): [N/mm2]
        fywk (float): [N/mm2]
        θ (float): Τhe angle between the concrete compression strut and the beam axis is the angle
            between the concrete compression strut and the beam axis [rad]
        αcw (float): Coefficient taking account of the state of the stress in the compression chord


    Returns:
        VRdmax (dict):

        Result value in [N], unless specified otherwise. Intermediate results are always 'N-mm'

        Given using the expression

        .. math::
            V_{Rd,max} = \dfrac{α_{cw}\cdot b_w \cdot z \cdot ν_1 \cdot f_{cd}}{\cot θ + \\tan θ}


    """
    if units == 'N-mm-rad':
        pass
    elif units == 'kN-m-rad':
        bw *= 1000
        d *= 1000
        fck *= 0.001
        fyk *= 0.001
        fywk *= 0.001
    else:
        pass

    _VRdmax = {}

    z = 0.9 * d
    fcd = fck / γc

    if fywk < 0.8 * fyk:
        if fck <= 60:
            v1 = 0.6
        else:
            v1 = max(0.5, 0.9 - fck / 200)
    else:
        v1 = 0.6 * (1 - fck / 250)

    _VRdmax['z'] = z
    _VRdmax['fcd'] = fcd
    _VRdmax['v1'] = v1

    _VRdmax['value'] = αcw * bw * z * v1 * fcd / (math.tan(θ) + 1 / math.tan(θ))

    if units == 'N-mm-rad':
        pass
    elif units == 'kN-m-rad':
        _VRdmax['value'] *= 0.001
    else:
        pass

    return _VRdmax


def VRds(nw, diaw, d, fyk, fywk, θ, s, γs=1.15, units='N-mm-rad'):
    """ Shear resistance for members with vertical shear reinforcement :math:`V_{Rd,s}` [N]

    Units are 'N-mm-rad', unless specified otherwise. Alternative options are: 'kN-m-rad'

    .. highlight:: python
    .. code-block:: python
       :linenos:

        vrds = shear.VRds(nw=2.,
                          diaw = 8,
                          d=539,
                          fyk=500.,
                          fywk=500.,
                          θ=np.pi/4,
                          s = 169,
                          γs = 1.15,
                          units='N-mm-rad')

    Args:
        nw (float): number of hoop legs.
        diaw (float): hoop bars diameter. [mm]
        d (float): Effective depth of the cross-section. [mm]
        fyk (float): [N/mm2]
        fywk (float): [N/mm2]
        θ (float): Τhe angle between the concrete compression strut and the beam axis is the angle
            between the concrete compression strut and the beam axis [rad]
        s (float): the spacing of the stirrups [mm]


    Returns:
        VRds (dict):

        Result value in [N], unless specified otherwise. Intermediate results are always 'N-mm'

        Given using the expression

        .. math::
            V_{Rd,max} = \dfrac{A_{sw}}{s} \cdot z \cdot f_{ywd} \cdot \cot θ

    """
    if units == 'N-mm-rad':
        pass
    elif units == 'kN-m-rad':
        diaw = 1000. * diaw
        d = 1000. * d
        fyk = 0.001 * fyk
        fywk = 0.001 * fywk
        s = 1000. * s
    else:
        pass

    _VRds = {}

    z = 0.9 * d
    Asw = nw * math.pi * diaw ** 2 / 4

    if fywk < 0.8 * fyk:
        fywd = min(0.8 * fywk, fywk / γs)
    else:
        fywd = fywk / γs

    _VRds['Asw'] = Asw
    _VRds['fywd'] = fywd

    _VRds['value'] = (Asw / s) * z * fywd * (1. / math.tan(θ))

    if units == 'N-mm-rad':
        pass
    elif units == 'kN-m-rad':
        _VRds['value'] *= 0.001
    else:
        pass

    return _VRds



# -*- coding: utf-8 -*-
from rmtk.vulnerability.common import utils
import csv
import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize

def assign_Mw_Tg(input_file, gmrs, Mw_multiplier, damping, filter):

    if filter == 'TRUE':
        Mw, Tg, record_name = [],[],[]
        with open(input_file, 'rU') as f:
            data = csv.reader(f)
            for line in data:
                record_name.append(line[0])
                Mw.append(float(line[1]))
                Tg.append(compute_predominant_period(record_name[-1], gmrs, damping))

        return [record_name, Mw, Tg, Mw_multiplier]
    else:
        return ['None']

def compute_predominant_period(gmr_name, gmrs, damping):

    igmr = gmrs['name'].index(gmr_name)
    T = np.arange(0.01,2,0.01)
    Sv = utils.calculate_iml(igmr, gmrs, 'Sv', T, damping)
    Tg = T[Sv==max(Sv)]

    return Tg[0]

def calculate_fragility_model(PDM,gmrs,gmr_info,IMT,msa,damage_model,T,damping, method):
    
    fragility_model = MSA_post_processing(PDM,gmrs,gmr_info,IMT,msa,damage_model,T,damping, method,0)

    return fragility_model

def calculate_fragility_model_damaged(PDMs,gmrs,gmr_info,IMT,msa,damage_model,T,damping, method):

    fragility_model = {}
    for iDS in range(1,len(PDMs)+1):
        fragility_model['iDS'+str(iDS)] = MSA_post_processing(PDMs['iDS'+str(iDS)],gmrs,np.array(gmr_info['iDS'+str(iDS)]),IMT,msa,damage_model,T,damping, method, iDS)

    return fragility_model

def MSA_post_processing(PDM,gmrs,gmr_info,IMT,msa,damage_model,T,damping, method,initial_damage):
    
    Nrecords = msa['records per bin']
    Nstripes = msa['n. bins']
    no_gmrs = Nrecords*Nstripes
    imls = []
    for index in np.arange(0,no_gmrs,Nrecords):
        igmr = gmr_info[index,0]
        scaling = gmr_info[index,1]
        imls.append(utils.calculate_iml(int(igmr), gmrs, IMT, T, damping)*scaling)
    
#    plt.plot(imls[0],Sds['iDS1'],marker='o', linestyle='None')
#    plt.ylim((0,0.5));
        
    no_assets = np.sum(PDM, axis=1, dtype=float)
    cumPDM = np.fliplr(np.cumsum(np.fliplr(PDM), axis=1))
    cumPDM = np.transpose(cumPDM)
    cumPDM = cumPDM/no_assets

    logmeans, logstddev = [], []
    I = np.argsort(imls, axis=0)
    imls = np.array(imls)
    imls = imls[I]
    if method == 'least squares':
        for iDS in range(initial_damage,len(damage_model['damage_states'])):
            cumPDM[iDS+1] = cumPDM[iDS+1][I]
            solution, _ = optimize.leastsq(utils.residual_lognormal_dist,
                                           [0.1, 0.6],
                                           args=(imls, cumPDM[iDS+1]))
            
            logmeans.append(solution[0])
            logstddev.append(solution[1])

    elif method == 'max likelihood':
        # the imls and the PDM are sorted to be used in the maximum likelihood approach
        # the PDM should be a list of integers in this approach
#        I = np.argsort(imls, axis=0)
#        imls = np.array(imls)
#        imls = imls[I]
        PDM_mle, PoE = [], []
        for iDS in range(initial_damage,len(damage_model['damage_states'])):
            cumPDM[iDS+1] = cumPDM[iDS+1][I]
            temp = cumPDM[iDS+1]*no_assets
            temp2 = temp.astype(int)
            PDM_mle = temp2.tolist()
            solution = utils.mle(PDM_mle,imls,no_assets,1)
            logmeans.append(solution[0])
            logstddev.append(solution[1])
    
    Rsquare = utils.calculate_correlation(logmeans,logstddev,cumPDM[initial_damage:],imls)

    fragility_model = {'damage_states': None, 'logmean': None,'logstddev': None, 'IMT': None}
    fragility_model['damage_states'] = damage_model['damage_states'][initial_damage:]
    fragility_model['logmean'] = logmeans
    fragility_model['logstddev'] = logstddev
    fragility_model['IMT'] = IMT
    fragility_model['Rsquare'] = Rsquare
    fragility_model['PoE points'] = cumPDM[initial_damage+1:]
    fragility_model['imls'] = imls

    return fragility_model

def plot_fragility_model(fragility_model,damage_model,minIML, maxIML):

    for iDS in range(1,len(damage_model['damage_states'])):
        utils.plot_fragility_model(fragility_model['iDS' + str(iDS)],minIML, maxIML)

def save_mean_fragility(fragility_model,damage_model,tax,output_type,output_path,minIML, maxIML):

    for iDS in range(1,len(damage_model['damage_states'])):
        taxonomy = tax+'_iDS'+str(iDS)
        utils.save_mean_fragility(taxonomy, fragility_model['iDS'+str(iDS)], minIML, maxIML, output_type, output_path)

def define_2MSA_parameters(no_bins,no_rec_bin,record_scaled_folder,filter_aftershocks):

    msa = {}; 
    msa['n. bins']=no_bins; 
    msa['records per bin']=no_rec_bin; 
    msa['input folder']=record_scaled_folder; 
    msa['filter'] = filter_aftershocks

    return msa


from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import orm
from sqlalchemy import Table, Column, Integer, String, Date, MetaData, Float, ForeignKey
from sqlalchemy import func

import pandas as pd

from .sql_classes import Base
from .sql_classes import Node, NodesCase, \
                Section, SectionsCase, \
                Element, ElementsCase, \
                Slab, SlabsCase, \
                NodeSlabConn, NodeSlabConnCase, \
                BeamSlabConn, BeamSlabConnCase, \
                Material, MaterialsCase, \
                BeamReinforcement, BeamReinforcementsCase, \
                ColumnReinforcement, ColumnReinforcementsCase, \
                ModFactor, ModFactorsCase, \
                BuildingIdentity


def create_db(excel_filename, db_filename):
    engine = create_engine(f'sqlite:///{db_filename}', echo=True)

    myBase = Base
    myBase.metadata.create_all(engine)

    Session = orm.sessionmaker(bind=engine)
    session = Session()

    # Load Excel file
    input_data = dict()
    xls = pd.ExcelFile(excel_filename)
    for sheet_name in xls.sheet_names:
        input_data[sheet_name] = pd.read_excel(excel_filename, sheet_name=sheet_name)

    # Create nodes_cases table
    nodes_cases_list = input_data['nodes'].CaseId.unique().tolist()
    for nc in nodes_cases_list:
        nodes_case = NodesCase(Id=nc)
        session.add(nodes_case)
    session.commit()

    # Create sections_cases table
    sections_cases_list = input_data['sections'].CaseId.unique().tolist()
    for sc in sections_cases_list:
        sections_case = SectionsCase(Id=sc)
        session.add(sections_case)
    session.commit()

    # Create elements_cases table
    elements_cases_list = input_data['elements'].CaseId.unique().tolist()
    for ec in elements_cases_list:
        elements_case = ElementsCase(Id=ec)
        session.add(elements_case)
    session.commit()

    # Create slabs_cases table
    slabs_cases_list = input_data['slabs'].CaseId.unique().tolist()
    for sc in slabs_cases_list:
        slabs_case = SlabsCase(Id=sc)
        session.add(slabs_case)
    session.commit()

    # Create node_slab_conns_cases table
    node_slab_conns_cases_list = input_data['node_slab_conns'].CaseId.unique().tolist()
    for nsc in node_slab_conns_cases_list:
        node_slab_conns_case = NodeSlabConnCase(Id=nsc)
        session.add(node_slab_conns_case)
    session.commit()

    # Create beam_slab_conns_cases table
    beam_slab_conns_cases_list = input_data['beam_slab_conns'].CaseId.unique().tolist()
    for bsc in beam_slab_conns_cases_list:
        beam_slab_conns_case = BeamSlabConnCase(Id=bsc)
        session.add(beam_slab_conns_case)
    session.commit()

    # Create materials_cases table
    materials_cases_list = input_data['materials'].CaseId.unique().tolist()
    for mc in materials_cases_list:
        materials_case = MaterialsCase(Id=mc)
        session.add(materials_case)
    session.commit()

    # Create beam_reinforcements_cases table
    beam_reinforcements_cases_list = input_data['beam_reinforcements'].CaseId.unique().tolist()
    for brc in beam_reinforcements_cases_list:
        beam_reinforcements_case = BeamReinforcementsCase(Id=brc)
        session.add(beam_reinforcements_case)
    session.commit()

    # Create column_reinforcements_cases table
    column_reinforcements_cases_list = input_data['column_reinforcements'].CaseId.unique().tolist()
    for crc in column_reinforcements_cases_list:
        column_reinforcements_case = ColumnReinforcementsCase(Id=crc)
        session.add(column_reinforcements_case)
    session.commit()

    # Create mod_factors_cases table
    mod_factors_cases_list = input_data['mod_factors'].CaseId.unique().tolist()
    for mfc in column_reinforcements_cases_list:
        mod_factors_case = ModFactorsCase(Id=mfc)
        session.add(mod_factors_case)
    session.commit()

    # Get data from excel file sheets
    con = engine.connect()
    for sheet_name in xls.sheet_names:
        input_data[sheet_name].to_sql(sheet_name, con, index=False, if_exists="append")
    # con.commit()
    con.close()

    session.close()


# create_db(excel_filename=r'D:\phd_closure\files\ch3\designs\all_building_models_sqlite_v1.xlsm',
#           db_filename='test_building_models.db')




import math
import numpy as np
import matplotlib.pyplot as plt
from sympy.solvers import solve
from sympy import Symbol


fwc = 1.5       # MPa
l = 6.0         # m
h = 3.0         # m
t = 0.2         # m
ρ = 1           # %

kw = 1.71 * fwc * (80 + h / t)
γcr = 0.09 / (math.pow(fwc, 0.5) * (80 + h / t))
τcr = kw * γcr

τu = 0.22 * math.pow(fwc, 0.5)
γu = 4.545 * γcr

print('kw = {0:.2f}MPa'.format(kw))
print('γcr = {0:.6f}'.format(γcr))
print('τcr = {0:.4f}MPa'.format(τcr))

print('γu = {0:.6f}'.format(γu))
print('τu = {0:.4f}MPa'.format(τu))

def τ(γ: float, τu: float, γu: float, ρ: float):
    x1 = (γ / γu) -1
    x2 = np.power(ρ, 3.82)
    return τu * (1 - 0.24 * np.sqrt( x1 / x2))

τ002 = τ(0.02, τu, γu, ρ)
print('τ0.02 = {0:.6f}'.format(τ002))

x1 = [0, γcr, γu]
y1 = [0, τcr, τu]

x2 = np.linspace(γu, 0.02, 50)
y2 = τ(x2, τu, γu, ρ)

Area = np.trapz(y2, x2)
print('Area = {0:.6f}'.format(Area))


γtmp = Symbol('γtmp')


γD = solve((τ002 * (0.02 - γtmp) + 0.5 * (γtmp - γu) * (τ002 + τu) - Area), γtmp)[0]
print('γD = {0:.6f}'.format(γD))

# τ002 * (0.02 - γD) + 0.5 * (γD - γu) * (τ002 + τu)


# υ = Symbol('υ')
# μθV=solve((((h-x)/(2*Ls))*min(N/1000, 0.55*b*d*fc/1000) + (1-0.05*υ) * (0.16*max(0.5, 100*ρtot)*(1-0.16*min(5,αs))*math.pow(fc/1000, 0.5)*b*d + Vw/1000)*1000-My/Ls),υ)[0]
#
# print(μθV)

xlin = [0, γcr, γu, γD, 0.02]
ylin = [0, τcr, τu, τ002, τ002]


plt.plot(x1, y1, lw=2)
plt.plot(x2, y2, lw=2)
plt.plot(xlin, ylin, lw=2)
#plt.axis([0, 1.2 * 100 * γu, 0, 1.2 * τu])
plt.axis([0, 0.021, 0, 1.1 * τu])
plt.ylabel('τ (MPa)')
plt.xlabel('γ')
plt.title('τ vs γ')

# plt.savefig( 'myfig.png' )
# plt.show()


kw_BC = (τu - τcr) / (γu - γcr)
kw_CD = (τ002 - τu) / (γD - γu)

print('kw_BC = {0:.2f}'.format(kw_BC))
print('kw_CD = {0:.2f}'.format(kw_CD))

Aw = l * t
α = np.arctan(h/l)  # Η γωνία με την οριζόντιο

EsAs_AB = 1000 * Aw * kw / (math.sin(α) * math.cos(α) * math.cos(α))
EsAs_BC = 1000 * Aw * kw_BC / (math.sin(α) * math.cos(α) * math.cos(α))
EsAs_CD = 1000 * Aw * kw_CD / (math.sin(α) * math.cos(α) * math.cos(α))

print('AsAs_AB = {0:.2f}'.format(EsAs_AB))
print('AsAs_BC = {0:.2f}'.format(EsAs_BC))
print('AsAs_CD = {0:.2f}'.format(EsAs_CD))

d = math.sqrt(math.pow(h,2) + math.pow(l,2))
dA = d
dB = math.sqrt(math.pow(h,2) + math.pow(l - h * γcr, 2))
dC = math.sqrt(math.pow(h,2) + math.pow(l - h * γu, 2))
dD = math.sqrt(math.pow(h,2) + math.pow(l - h * γD, 2))
dE = math.sqrt(math.pow(h,2) + math.pow(l - h * 0.02, 2))

print('dA = {0:.4f}'.format(dA))
print('dB = {0:.4f}'.format(dB))
print('dC = {0:.4f}'.format(dC))
print('dD = {0:.4f}'.format(dD))
print('dE = {0:.4f}'.format(dE))

δA = 0.0
δB = - (dB - dA)
δC = - (dC - dA)
δD = - (dD - dA)
δE = - (dE - dA)

print('δA = {0:.4f}'.format(δA))
print('δB = {0:.4f}'.format(δB))
print('δC = {0:.4f}'.format(δC))
print('δD = {0:.4f}'.format(δD))
print('δE = {0:.4f}'.format(δE))

FA = 0
FB = EsAs_AB * δB / d
FC = EsAs_BC * (δC - δB) / d + FB
FD = EsAs_CD * (δD - δC) / d + FC
FE = FD

print('FA = {0:.4f}'.format(FA))
print('FB = {0:.4f}'.format(FB))
print('FC = {0:.4f}'.format(FC))
print('FD = {0:.4f}'.format(FD))
print('FE = {0:.4f}'.format(FE))

XFδ = [δA, δB, δC, δD, δE]
YFδ = [FA, FB, FC, FD, FE]

plt.plot(XFδ, YFδ, lw=2)
plt.axis([0, 1.1 * δE, 0, 1.1 * FC])
plt.ylabel('F (kN)')
plt.xlabel('δ (m)')
plt.title('F vs δ')

plt.show()


import numpy as np
from dataclasses import dataclass

from ...raw.ch3.seismic_action import spectra as spec_raw


@dataclass
class SpectraEc8:
    """Eurocode 8 response spectra

    .. note::
        If αgR values are given in g, displacements and velocities should be multiplied with 9.81

    .. uml::

        class SpectraEc8 {
        .. attributes ..
        + αgR (float)
        + γI (float)
        + ground_type (str)
        + spectrum_type (int)
        + η (float)
        + q (float)
        + β (float)
        .. properties ..
        + dg()
        + getAllSpectra0to4()
        .. methods ..
        + Se(T) (float)
        + SDe(T) (float)
        + Sd(T) (float)
        }

    Attributes:
        αgR (float): reference peak ground acceleration on type A ground
        γI (float): importance factor
        ground_type (str): Ground type (A, B, C, D or E)
        spectrum_type (int): Spectrum type 1 or 2
        η (float): value of the damping correction factor
        q (float): behaviour factor
        β (float): lower bound factor for the horizontal design spectrum. Recommended value for β is 0.2

    """
    αgR: float
    γI: float
    ground_type: str
    spectrum_type: int
    η: float = 1.0
    q: float = 1.0
    β: float = 0.2

    def __post_init__(self):
        self.αg = self.γI * self.αgR
        self.S = spec_raw.S(ground_type=self.ground_type, spectrum_type=self.spectrum_type)
        self.TB = spec_raw.TB(ground_type=self.ground_type, spectrum_type=self.spectrum_type)
        self.TC = spec_raw.TC(ground_type=self.ground_type, spectrum_type=self.spectrum_type)
        self.TD = spec_raw.TD(ground_type=self.ground_type, spectrum_type=self.spectrum_type)

    def Se(self, T):
        """
        Args:
            T(float): Period

        Returns:
            float: The elastic acceleration response spectrum

        """
        return spec_raw.Se(T, self.αg, self.S, self.TB, self.TC, self.TD, self.η)

    def SDe(self, T):
        """
        Args:
            T(float): Period

        Returns:
            float: The elastic displacement response spectrum

        """
        return spec_raw.SDe(T, self.Se(T))

    @property
    def dg(self):
        """float: Design ground displacement"""
        return spec_raw.dg(self.αg, self.S, self.TC, self.TD)

    def Sd(self, T):
        """
        Args:
            T(float): Period

        Returns:
            float: Design spectrum for elastic analyses

        """
        return spec_raw.Sd(T, self.αg, self.S, self.TB, self.TC, self.TD, self.q, self.β)

    @property
    def getAllSpectra0to4(self):
        """ dict: A dictionary of numpy arrays with spectral values in the range 0-4sec"""
        _T_range = np.linspace(0.01, 4, 400)

        _Se = self.Se(_T_range)
        _SDe = self.SDe(_T_range)
        _Sv = _Se / (2 * np.pi / _T_range)
        _Sd = self.Sd(_T_range)

        _T_range = np.append(0., _T_range)
        _Se = np.append(self.αg * self.S, _Se)
        _SDe = np.append(0., _SDe)
        _Sv = np.append(0., _Sv)
        _Sd = np.append(self.αg * self.S * 2.0 / 3.0, _Sd)

        _data = {'T': _T_range,
                 'Se': _Se,
                 'Sv': _Sv,
                 'SDe': _SDe,
                 'Sd': _Sd}

        return _data


class ExampleClass(object):
    """The summary line for a class docstring should fit on one line.

    If the class has public attributes, they may be documented here
    in an ``Attributes`` section and follow the same formatting as a
    function's ``Args`` section. Alternatively, attributes may be documented
    inline with the attribute's declaration (see __init__ method below).

    Properties created with the ``@property`` decorator should be documented
    in the property's getter method.

    Attributes:
        attr1 (str): Description of `attr1`.
        attr2 (:obj:`int`, optional): Description of `attr2`.

    """

    def __init__(self, param1, param2, param3):
        """Example of docstring on the __init__ method.

        The __init__ method may be documented in either the class level
        docstring, or as a docstring on the __init__ method itself.

        Either form is acceptable, but the two should not be mixed. Choose one
        convention to document the __init__ method and be consistent with it.

        Note:
            Do not include the `self` parameter in the ``Args`` section.

        Args:
            param1 (str): Description of `param1`.
            param2 (:obj:`int`, optional): Description of `param2`. Multiple
                lines are supported.
            param3 (:obj:`list` of :obj:`str`): Description of `param3`.

        """
        self.attr1 = param1
        self.attr2 = param2
        self.attr3 = param3  #: Doc comment *inline* with attribute

        #: list of str: Doc comment *before* attribute, with type specified
        self.attr4 = ['attr4']

        self.attr5 = None
        """str: Docstring *after* attribute, with type specified."""

    @property
    def readonly_property(self):
        """str: Properties should be documented in their getter method."""
        return 'readonly_property'

    @property
    def readwrite_property(self):
        """:obj:`list` of :obj:`str`: Properties with both a getter and setter
        should only be documented in their getter method.

        If the setter method contains notable behavior, it should be
        mentioned here.
        """
        return ['readwrite_property']

    @readwrite_property.setter
    def readwrite_property(self, value):
        value

    def example_method(self, param1, param2):
        """Class methods are similar to regular functions.

        Note:
            Do not include the `self` parameter in the ``Args`` section.

        Args:
            param1: The first parameter.
            param2: The second parameter.

        Returns:
            True if successful, False otherwise.

        """
        return True

    def __special__(self):
        """By default special members with docstrings are not included.

        Special members are any methods or attributes that start with and
        end with a double underscore. Any special member with a docstring
        will be included in the output, if
        ``napoleon_include_special_with_doc`` is set to True.

        This behavior can be enabled by changing the following setting in
        Sphinx's conf.py::

            napoleon_include_special_with_doc = True

        """
        pass

    def __special_without_docstring__(self):
        pass

    def _private(self):
        """By default private members are not included.

        Private members are any methods or attributes that start with an
        underscore and are *not* special. By default they are not included
        in the output.

        This behavior can be changed such that private members *are* included
        by changing the following setting in Sphinx's conf.py::

            napoleon_include_private_with_doc = True

        """
        pass

    def _private_without_docstring(self):
        pass
